/* Regression Test */

pre {
	margin: 0;
    white-space: pre-wrap;
}

.rt-table-fixed tbody {
	max-height: 500px;
	overflow-y: auto;
	display: block;
}

.rt-table-fixed tr {
	display: block;
}

.rt-changes {
	margin-bottom: 500px;
}

.rt-pages {
	display: inline-block;
}

.rt-expansion {
	display: block;
	position: relative;
}

.btnCollapse {
	display: block;
	position: absolute;
	top: 0;
	right: 0;
}

.nav-link.rt-changed {
	color: red;
}
.nav-link.rt-gold {
	color: gold;
}

.rt-output {
	white-space: pre-wrap;
	word-break: break-all;
	overflow-wrap: break-word;
	font-family: monospace;
	font-size: 11px;
	display: block;
	max-height: 500px;
	overflow-y: auto;
}
.rt-input:before {
	content: "Input: ";
	font-style: normal;
	font-weight: bold;
}
.rt-input {
	font-size: 11px;
	font-style: italic;
	margin: 1ex;
}

ul.rt-gold {
	margin-top: 1ex;
}

ul.rt-gold:before {
    content: "Golds: ";
    font-style: normal;
    font-weight: bold;
}

input.rt-gold-input-box {
    width: 75%;
}

select.selectDiffMode {
    width: 100px;
    display: inline-block;
}

.rt-count {
	font-size: 70%;
	margin-left: 20px;
}

.c-fst-b, .c-cg-b, .c-ap-l {
	color: #880;
}
.c-fst-p, .c-cg-p, .c-t-t, .c-ap-t {
	color: #008;
}
.c-fst-s, .c-cg-s {
	color: #080;
}
.c-fst-pl, .c-ap-d {
	color: #ccc;
}
.c-cg-m, .c-ap-u {
	color: #800;
}
.c-cg-sc {
	color: #808;
}
.c-cg-ps {
	color: #088;
}
.c-cg-t {
	color: #44F;
}
.c-ap-lu {
    white-space: nowrap;
}

code {
	white-space: pre-wrap;
}

del, ins {
	text-decoration: none;
	font-weight: bold;
}

ins {
	background-color: #cfc;
}

del {
	background-color: #fcc;
}

div {
	-moz-tab-size: 4;
	tab-size: 4;
}

/* Bootstrap overrides */

.toast {
	min-width: 200px;
}

.nav-tabs {
	background-color: #eee;
}

.nav-tabs .nav-link.active {
	color: #fff;
	background-color: #007bff;
	border-color: #007bff;
}
